<?php

/**
 * @file
 * Admin page callback file for the wow module.
 */

/**
 * Form builder; The API settings form.
 *
 * @ingroup forms
 * @see system_settings_form()
 */
function wow_admin_settings($form, &$form_state) {
  // Get the list of options to populate the region dropdown.
  $options_region = wow_regions();

  $locked = (bool) variable_get('wow_region', FALSE);

  // If we have a value for the region dropdown from $form_state['values'] we use
  // this both as the default value for the region dropdown and also as a
  // parameter to pass to the function that retrieves the options for the
  // realm dropdown.
  $region = isset($form_state['values']['wow_region']) ? $form_state['values']['wow_region'] : variable_get('wow_region', key($options_region));
  $locale = isset($form_state['values']['wow_locale']) ? $form_state['values']['wow_locale'] : variable_get('wow_locale', '');

  if (!$locked && !isset($form_state['values']['wow_region'])) {
    // Build default values depending on the website settings.
    switch (variable_get('site_default_country')) {
      default:
      case 'US':
      case 'MX':
      case 'BR':
        $region = 'us';
        break;

      case 'GB':
      case 'ES':
      case 'FR':
      case 'RU':
      case 'DE':
      case 'PT':
        $region = 'eu';
        break;

      case 'KR':
        $region = 'kr';
        break;

      case 'TW':
        $region = 'tw';
        break;

      case 'CN':
        $region = 'cn';
        break;
    }

    $locales = wow_api_info($region, 'locales');
    $language = language_default('language');

    if (!array_key_exists($language, $locales)) {
      $language = key($locales);
    }

    $locale = $locales[$language];
  }

  $form['api'] = array(
    '#type' => 'fieldset',
    '#title' => t('API settings'),
    '#description' => t('The default API settings for the website.'),
  );
  $form['api']['wow_region'] = array(
    '#type' => 'select',
    '#title' => t('Region'),
    '#options' => $options_region,
    '#default_value' => $region,
    // Bind an ajax callback to the change event (which is the default for the
    // select form type) of the region dropdown. It will replace the locale
    // dropdown when rebuilt.
    '#ajax' => array(
      'callback' => 'wow_admin_region_callback',
      'wrapper' => 'api-realm',
    ),
    '#disabled' => $locked,
  );

  $form['api']['wow_locale'] = array(
    '#type' => 'select',
    '#title' => t('Locale'),
    // The entire enclosing div created here gets replaced when wow_region
    // is changed.
    '#prefix' => '<div id="api-realm">',
    '#suffix' => '</div>',
    // When the form is rebuilt during ajax processing, the $selected variable
    // will now have the new value and so the options will change.
    '#options' => wow_locale_dropdown_options($region),
    '#default_value' => $locale,
    '#description' => t('This is the default locale used for API calls.'),
  );
  $form['api']['wow_cdn'] = array(
    '#type' => 'value',
    '#value' => wow_api_info($region, 'content'),
  );

  if ($locked) {
    $form['api']['wow_region']['#description'] = t('To reset the region settings, you must disable and uninstall the WoW module. This operation will erase all your data.');
  }
  else {
    $form['api']['#description'] .= ' ' . t('Once you saved your settings, region will be locked. If you want to change again the region, you must disable and uninstall the WoW module.');
  }

  return system_settings_form($form);
}

/**
 * Selects just the locale dropdown to be returned for re-rendering.
 *
 * Since the controlling logic for populating the form is in the form builder
 * function, all we do here is select the element and return it to be updated.
 *
 * @return array
 *   The locale dropdown.
 */
function wow_admin_region_callback($form, $form_state) {
  return $form['api']['wow_locale'];
}

/**
 * Helper function to populate the locale dropdown.
 */
function wow_locale_dropdown_options($region) {
  $locales = array_values(wow_api_info($region, 'locales'));
  return drupal_map_assoc($locales);
}

/**
 * Helper function to populate the realm dropdown. This call the battle.net API.
 */
function wow_realm_dropdown_options($region = NULL) {
  if (is_null($region)) {
    $region = variable_get('wow_region', 'us');
  }

  if ($cache = cache_get('wow_realms:' . $region)) {
    $realms = $cache->data;
  }
  else {
    $response = wow_realm_fetch($region);

    foreach ($response->realms as $realm) {
      $realms[$realm->name] = $realm->name;
    }

    cache_set('wow_realms:' . $region, $realms, 'cache', CACHE_TEMPORARY);
  }

  return $realms;
}

/**
 * Return the wow regions supported by the API.
 */
function wow_regions() {
  $regions = array();

  foreach (wow_api_info() as $region => $info) {
    $regions[$region] = $info['name'];
  }

  return $regions;
}

/**
 * Menu callback; Displays wow admin dashboard.
 */
function wow_admin() {
  return '';
}